/**
*@ author Sebastian Seyrich, Markus Mattes
*/

import java.rmi.Remote;
import java.rmi.server.*;
import java.rmi.RemoteException;
import java.net.URL;

/**
 *Administrates a game, includes remotmethods for:
 *	- throw a chip 
 *	- log out 
 *	- receive a message
 */

public class Game extends UnicastRemoteObject implements GameInterface{
    
    private RemoteClientControlInterface playerA;
    private RemoteClientControlInterface playerB;
    private RemoteClientControlInterface activePlayer;
    private int counter;
    private GameServerInterface gameServer;
    int index;
    private String messageA;
    private String messageB;
    private int[][] score;
    
    
    public Game(GameServerInterface gsi) throws RemoteException{
	super();
	index=0;
	score = new int[7][6];
	gameServer = gsi;
    }	
    
    /**
     *registers a player on his RemoteClientControl-Object.
     *true yields succsesful
     *false not succsesful
     */
    public boolean registerPlayer(RemoteClientControlInterface playerRCC) throws RemoteException{
	System.out.println("Game: registerPlayer() aufgerufen");
	if(playerA!=null&&playerB!=null)
	    System.out.println("Game beide Spieler ungleich null");
	
	if(playerA == null){
	    playerA = playerRCC; 
	    System.out.println("Game: registerPlayer() playerA registriert ");
	    if(playerA!=null&&playerB!=null)
		System.out.println("\tGame beide Spieler ungleich null"); 
	    return true;
	} 
	if(playerB == null){ 
	    System.out.println("Game: registerPlayer() versuche playerB zu registriert");  
	    playerB = playerRCC;
	    activePlayer = playerA;
	    playerA.setActive(true);
	    System.out.println("Game: registerPlayer() playerB registriert");  
	    return true;
	} 
	return false;
    }
    
    
    
    /**
     *Game will know about a new choice 
     *Sould check if the player is part of the game and if the choice was legal
     *Initiate the graphical update
     */
    public synchronized void makeYourChoice(int column,RemoteClientControlInterface playerRCC) throws RemoteException{
	System.out.println("Game: makeYourChoice(), aufgerufen");
	
	if(playerRCC.equals(activePlayer)){
	    if(turn(column)){
		activePlayer.setActive(false);
		nextPlayer();
		activePlayer.setActive(true);
	    }
	    playerA.updateColumn(column,score[column]);
	    playerB.updateColumn(column,score[column]);
	}
    }
    
    /**
     *Will check if a new chip in the panel will finish the game
     */
    private boolean turn(int column) throws RemoteException{
	System.out.println("Am Anfang von Turn");
	if (counter==42){
	    activePlayer.bye(0);nextPlayer();activePlayer.bye(0);return false;
	}
	if(column<0 || column>=score.length || score[column][score[column].length-1]!=0)
	    return false;
	int row=0;
	int ident=0;
	counter++;
	if(activePlayer==playerA)
	    ident=1;
	if(activePlayer==playerB)
	    ident=2;	
	for(int k=0;k<score[column].length;k++){
	    if (score[column][k]!=0){}
	    else {
		score[column][k]=ident;
		row = k;
		k=score[column].length;			
	    }
	    
	}	
	
	//Search for three chips under the last input chip
	if ((row-1)>=0&&score[column][row-1]==ident)
	    if ((row-2)>=0&&score[column][row-2]==ident)
		if ((row-3)>=0&&score[column][row-3]==ident)
		    {activePlayer.bye(2);nextPlayer();activePlayer.bye(1);return false;}
	
	//Search for three chips diagonal left down and right up
	if ((column-1)>=0&&(row-1)>=0&&score[column-1][row-1]==ident)							
	    if ((column-2)>=0&&(row-2)>=0&&score[column-2][row-2]==ident)
		if ((column-3)>=0&&(row-3)>=0&&score[column-3][row-3]==ident)
		    {activePlayer.bye(2);nextPlayer();activePlayer.bye(1);return false;}	
		else if((column+1)<7&&(row+1)<7&&score[column+1][row+1]==ident)
		    {activePlayer.bye(2);nextPlayer();activePlayer.bye(1);return false;}
		else if((column+1)<7&&(row+1)<7&&score[column+1][row+1]==ident)
		    if((column+2)<7&&(row+2)<7&&score[column+2][row+2]==ident)
			{activePlayer.bye(2);nextPlayer();activePlayer.bye(1);return false;}
		    else if((column+1)<7&&(row+1)<7&&score[column+1][row+1]==ident)
			if((column+2)<7&&(row+2)<7&&score[column+2][row+2]==ident)
			    if((column+3)<7&&(row+3)<7&&score[column+3][row+3]==ident)
				{activePlayer.bye(2);nextPlayer();activePlayer.bye(1);return false;}
	
	//Search for three chips diagonal right down and left up
	if ((column+1)<7&&(row-1)>=0&&score[column+1][row-1]==ident)							
	    if ((column+2)<7&&(row-2)>=0&&score[column+2][row-2]==ident)
		if ((column+3)<7&&(row-3)>=0&&score[column+3][row-3]==ident)
		    {activePlayer.bye(2);nextPlayer();activePlayer.bye(1);return false;}	
		else if((column-1)>=0&&(row+1)<7&&score[column-1][row+1]==ident)
		    {activePlayer.bye(2);nextPlayer();activePlayer.bye(1);return false;}
		else if((column-1)>=0&&(row+1)<7&&score[column-1][row+1]==ident)
		    if((column-2)>=0&&(row+2)<7&&score[column-2][row+2]==ident)
			{activePlayer.bye(2);nextPlayer();activePlayer.bye(1);return false;}
		    else if((column-1)>=0&&(row+1)<7&&score[column-1][row+1]==ident)
			if((column-2)>=0&&(row+2)<7&&score[column-2][row+2]==ident)
			    if((column-3)>=0&&(row+3)<7&&score[column-3][row+3]==ident)
				{activePlayer.bye(2);nextPlayer();activePlayer.bye(1);return false;}		
	
	
	//Search for three chips right and then left
	if ((column+1)<7&&score[column+1][row]==ident){
	    if ((column+2)<7&&score[column+2][row]==ident){
		if ((column+3)<7&&score[column+3][row]==ident){
		    activePlayer.bye(2);nextPlayer();activePlayer.bye(1);return false;
		}
		else if((column-1)>=0&&score[column-1][row]==ident){
		    activePlayer.bye(2);nextPlayer();activePlayer.bye(1);return false;
		}
	    }
	    
	    else if((column-1)>=0&&score[column-1][row]==ident){
		if((column-2)>=0&&score[column-2][row]==ident){
		    activePlayer.bye(2);nextPlayer();activePlayer.bye(1);return false;
		}
	    }
	}
	if((column-1)>=0&&score[column-1][row]==ident){
	    if((column-2)>=0&&score[column-2][row]==ident){
		if((column-3)>=0&&score[column-3][row]==ident){
		    activePlayer.bye(2);nextPlayer();activePlayer.bye(1);return false;
		}
	    }
	}

			
	return true;
	
    }
    
    
	private synchronized void nextPlayer(){
		if(activePlayer.equals(playerA))
			activePlayer = playerB;
		else
			activePlayer = playerA;		
		try{
			System.out.println("Game: nach nextPlayer() ist activePlayer: "+activePlayer.getName());
		}catch(RemoteException e){System.out.println("RemoteException ist aufgetreten:\n"+e.toString());}
	
	}


	/**
	*You have logged out 
	*game will be deleted at the gameserver 		
	*/
	public void logout(RemoteClientControlInterface playerRCC) throws RemoteException{
		if(playerRCC.equals(playerA)||playerRCC.equals(playerB)){
			playerB.bye(4);playerB=null;
			playerA.bye(4);playerA=null;
			System.out.println("Game: Der folgende Spieler hat sich ausgeloggt: "+playerRCC.getName());
			
			gameServer.logout(this);

		}
		else {
			System.out.println("Game: Der  folgende Spieler hat versucht sich auszuloggen:"+playerRCC.getName());
		}
	}
	
	protected RemoteClientControlInterface getPlayerA(){
		return playerA;
	}

	protected RemoteClientControlInterface getPlayerB(){
		return playerB;
	}

                                                    
}
